/*
 * detonate.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

#undef P_
#endif

/* Detonate torp */

/*
 * * Here we have another flaw.  Detonating other players torps can be a *
 * very quick way to die.  Why?  Because you always take some damage. *
 * Experienced players never detonate other players' torps.  Balance is *
 * really hard to obtain with this type of function.  Technically, a * player
 * could nearly continuously detonate torps (at least faster than * they
 * could be fired) and never be hurt, if I allowed less damage as * a
 * possible result.  So here it sits.
 */

void
detothers()
{
   register int    h, i;
   int             dx, dy;
   register struct torp *j;

   if (me->p_fuel < myship->s_detcost) {
      warning(0, "Not enough fuel to detonate", -1);
      return;
   }
   if (me->p_flags & PFWEP) {
      warning(0, "Weapons overheated", 35);
      return;
   }
   me->p_fuel -= myship->s_detcost;
   me->p_wtemp += myship->s_detcost / 5;

   for (h = 0; h < MAXPLAYER; h++) {
      if ((players[h].p_status == PFREE) || (h == me->p_no))
	 continue;
      for (i = h * MAXTORP; i < MAXTORP * (h + 1); i++) {
	 j = &torps[i];
	 if (friendlyTorp(j))
	    continue;
#ifdef MINES
	 if (mines && j->t_mine)
	    continue;		/* can't det mines */
#endif
	 if ((j->t_status == TMOVE) || (j->t_status == TSTRAIGHT)) {
	    dx = j->t_x - me->p_x;
	    dy = j->t_y - me->p_y;
	    if (ABS(dx) > DETDIST || ABS(dy) > DETDIST)	/* XXX */
	       continue;
	    if (dx * dx + dy * dy < DETDIST * DETDIST) {	/* fix, TC */
	       j->t_whodet = me->p_no;
	       j->t_status = TDET;
	    }
	 }
      }
   }
}
