/*
 * smessage.c
 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "packets.h"
#ifndef _NO_DEPEND
#   include "ntserv_protos.h"
#endif

#ifdef USE_SYS_PROTOS
#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

extern char *                 strstr P_((const char *, const char *));
extern int                    strncmp P_((const char *, const char *, size_t));
extern int                    fprintf P_((FILE *, const char *, ...));
extern int                    sscanf P_((const char *, const char *, ...));
extern FILE *                 fopen P_((const char *, const char *));
extern int                    fclose P_((FILE *));
extern size_t                 strftime P_((char *, size_t, const char *, const struct tm *));
extern struct tm *            localtime P_((const time_t *));
extern void                   perror P_((const char *));
extern time_t                 time P_((time_t *));

#undef P_
#endif

void
pmessage(str, recip, group, address)
   char           *str;
   int             recip;
   int             group;
   char           *address;
{
   /* Message from God */
   pmessage2(str, recip, group, address, 255);
}

void
pmessage2(str, recip, group, address, from)
   char           *str;
   int             recip;
   int             group;
   char           *address;
   unsigned char   from;
{
   struct message *cur;
   int             mesgnum;

   if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE) {
      mctl->mc_current = 0;
      mesgnum = 0;
   }
   cur = &messages[mesgnum];
   cur->m_no = mesgnum;
   cur->m_flags = group;
   cur->m_recpt = recip;
   cur->m_from = from;
   copy80(cur->m_data, "%-9s ", address, str);
   cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
   cur->args[0] = DINVALID; /* to prevent false sending with SPS_S_WARNING*/
#endif
   if ((group == MINDIV) && (recip == from) && 
       (strncmp(str, "::", 2) == 0)) {	
   /* Talking to yourself with a "::" is a secret back door */

      char            aBuf[150];

      if (strncmp(str, "::waitq", 7)==0) {
	 sprintf(aBuf, "There are %d people in the wait-queue.", status->count);
	 warning(1, aBuf, -1);
      } else if (strncmp(str, "::pcount", 8)==0) {
	 int             a, fed = 0, rom = 0, kli = 0, ori = 0, nob = 0;

	 for (a = 0; a < MAXPLANETS; a++) {
	    if (planets[a].pl_owner == FED)
	       fed++;
	    else if (planets[a].pl_owner == ROM)
	       rom++;
	    else if (planets[a].pl_owner == KLI)
	       kli++;
	    else if (planets[a].pl_owner == ORI)
	       ori++;
	    else if (planets[a].pl_owner == NOBODY)
	       nob++;
	 }
	 sprintf(aBuf, "Planet counts: Fed=%d, Rom=%d, Kli=%d, Ori=%d, Ind=%d.", fed, rom, kli, ori, nob);
	 warning(1, aBuf, -1);
      }
      else if (strncmp(str, "::help", 6)==0){
	 warning(1, "waitq, pcount, and help are all valid requests.", -1);
      }
      else if (strncmp(str, "::broke", 7)==0){
	 extern int	broke;
	 broke = !broke;
	 sprintf(aBuf, "20%% packet loss %s", broke?"on":"off");
	 warning(1, aBuf, -1);
      }
      else
	 warning(1, "waitq, pcount, and help are all valid requests.", -1);
   }
}

char *
getaddr2(flags, from, recip) 

   int flags;
   int from, recip;
{
   static char 		addrmesg[32];           /* increased, TSH 2/93 */
   struct player	*j = NULL;

   if(from < MAXPLAYER)
      j = &players[from];

   switch(flags) {
      case MALL:
	 if(!j)
	    (void) sprintf(addrmesg, "GOD->ALL");
	 else
	    (void) sprintf(addrmesg, " %c%c->ALL", 
	       j->p_mapchars[0], j->p_mapchars[1]);
	 break;
      case MGOD:
	 if(!j)
	    (void) sprintf(addrmesg, "GOD->GOD");
	 else
	    (void) sprintf(addrmesg, " %c%c->GOD",
	       j->p_mapchars[0], j->p_mapchars[1]);
	 break;
      case MTEAM:
	 if(!j)
	    (void) sprintf(addrmesg, "GOD->%s", teamshort[recip]);
	 else
	    (void) sprintf(addrmesg, " %c%c->%s", 
	       j->p_mapchars[0], j->p_mapchars[1], teamshort[recip]);
	 break;
      case MINDIV:
	 if(recip >= MAXPLAYER){
	    if(!j)
	       (void) sprintf(addrmesg, "GOD->GOD");
	    else
	       (void) sprintf(addrmesg, " %c%c->GOD",
		  j->p_mapchars[0], j->p_mapchars[1]);
	 }
	 else{
	    if(!j)
	       (void) sprintf(addrmesg, "GOD->%c%c",
		  teamlet[players[recip].p_team], shipnos[recip]);
	    else
	       (void) sprintf(addrmesg, " %c%c->%c%c",
		  j->p_mapchars[0], j->p_mapchars[1], 
		     teamlet[players[recip].p_team], shipnos[recip]);
	 }
	 break;
   }
   return addrmesg;
}
