/*
 * Startup program for ntserv.  Run it in the background.  It listens
 * for connections on all ports specified in PORTFILE and starts up new ntserv
 * process as needed.  It really should be started from inetd, but it
 * doesn't support that right now, and actually makes it easier to disable
 * the game whenever you want to just by killing this program.
 * 
 * (C) 1989 Brett McCoy
 * 
 * Program created 24 Oct 1989
 * 
 * Permission granted for anyone to use and modify this code as needed, just
 * don't pretend that you wrote it.
 */

/* #define SERVICES  */
#define TIMESTAMP

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <signal.h>
#include <syslog.h>
#ifdef SYSLOG
#include <syslog.h>
#endif

#include "version.h"
#include "patchlevel.h"


#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* ntstart.c */
void main P_((int argc, char **argv, char **envp));
void detach P_((void));
void handler P_((void));
void read_portfile P_((void));
void output_timestamp_diag P_((FILE *outfile, int ind));
void start_services P_((void));
void shutdownAll P_((void));

#ifdef USE_SYS_PROTOS
extern pid_t fork P_((void));
extern uid_t getuid P_((void));
extern uid_t geteuid P_((void));
extern void perror P_((const char *));
extern int atoi P_((const char *));
extern int printf P_((const char *,...));
extern int fprintf P_((FILE *, const char *,...));
extern unsigned int alarm P_((unsigned int));
extern int close P_((int));
extern int fflush P_((FILE *));
extern int shutdown P_((int s, int how));
extern int socket P_((int domain, int type, int protocol));
extern int connect P_((int s, struct sockaddr * name, int namelen));
extern int getpeername P_((int s, struct sockaddr * name, int *namelen));
extern pid_t wait3 P_((union wait *, int, struct rusage *));
extern int bind P_((int s, struct sockaddr * name, int namelen));
extern int listen P_((int s, int backlog));
extern char *inet_ntoa P_((/* ??? */));
extern time_t time P_((time_t *));
extern int dup2 P_((int, int));
extern int ioctl P_((int, int, ...));
extern int accept P_((int s, struct sockaddr *addr, int *addrlen));
extern int setsockopt P_((int s, int level, int optname, char *optval, int optlen));
extern void                   bzero P_((void *, size_t));
extern int                    sscanf P_((const char *, const char *, ...));
extern int                    fclose P_((FILE *));
extern int                    select P_((int width, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout));
extern int                    fputc P_((int, FILE *));
#endif

#undef P_

#ifdef CALVIN
#   define SERVERDIR 	"/home/calvin/ntacct/Netrek"
#endif

#ifdef SERVICES
#   define SERVICEFILE	".services"
#endif

/*
 * change these to point to wherever your ntserv program is and where you
 * want the startup logs to go.
 */
#define LOGFILE         "startup.log"
#define ERRFILE         "startup.err"

/* port/program configuration file */
#define PORTFILE	"./.ports"

/* this is used if port file can't be opened */
#define DEFAULT_PROG		"./ntserv"
#define DEFAULT_PROGNAME	"./Netrek"
#define DEFAULT_PORT		2592	

#define MAXPROG			16

static
struct progrecord {
   unsigned short	port;
   int			sock;
   char			prog[512], 
			progname[512],
			/* any special arguments */
			arg[4][64];
} prog[MAXPROG];

int	num_progs=0;

void
main(argc, argv, envp)

   int             argc;
   char          **argv, **envp;
{
   register	   i, j;
   int             sock;
   struct sockaddr_in addr;
   struct sockaddr_in naddr;
   struct hostent *hp;
   int             len, st;
   int             pid;
#ifndef TIMESTAMP
   time_t          curtime;
#endif
   char            host_name[100];
   extern void     handler();
   extern int      errno;
   int             optval;
   static struct linger ling = {0, 0};
   fd_set	   accept_fds;
   register struct progrecord	*pr;
   int		   p[2];
#ifdef TIMESTAMP
   int		   proc_stdout, proc_stderr;
   FILE		   *lfo, *efo;
#endif


#ifdef SYSLOG
   openlog("netrek",LOG_PID,LOG_USER);
#endif SYSLOG


   if (fork() != 0)
      exit(0);

#if defined(SERVERDIR)

   /* we'll put this in so can be started from boot */

   if (chdir(SERVERDIR) < 0) {
      perror("ntstart'chdir");
      exit(1);
   }

   /* if we're started as root set the id to the owner of 'ntstart' */

   if (getuid() == 0 || geteuid() == 0) {
      struct stat     sbuf;
      if (stat(argv[0], &sbuf) < 0) {
	 perror("ntstart'stat");
      } else {
	 setuid(sbuf.st_uid);
	 setgid(sbuf.st_gid);
      }
   }
#endif

   read_portfile();
   if(!num_progs) exit(0);

#ifndef TIMESTAMP
   /*
    * The first order of business is to detach this process from the
    * controlling terminal and then reopen the stdout and stderr to files for
    * keeping a log of what happens.
    */

   fclose(stdout);
   fclose(stderr);
#endif


   /*
    * set the child exit signal to point at our own routine
    */
   signal(SIGCHLD, handler);
   signal(SIGHUP, shutdownAll);
   signal(SIGTERM, shutdownAll);

#ifdef SERVICES
   /* 
    * Start any special programs defined in .services.  Mostly cleanup
    * of old logfiles
    */

   start_services();
#endif

#ifdef TIMESTAMP
   lfo = fopen(LOGFILE, "a");
   if(!lfo) { perror(LOGFILE); exit(1); }
   efo = fopen(ERRFILE, "a");
   if(!efo) { perror(ERRFILE); exit(1); }

   /* pipe stdout and stderr to a file descriptor we can read */
   if(pipe(p) < 0) { perror("pipe"); exit(1); }

   proc_stdout = p[0];
   dup2(p[1], 1);	/* we read on proc_stdout anything that was written
			   to stdout */

   if(pipe(p) < 0) { perror("pipe"); exit(1); }

   proc_stderr = p[0];
   dup2(p[1], 2);	/* we read on proc_stderr anything that was written
			   to stderr */
   
   /* If 'ntstart' exits, there's no one to read the pipe and it will 
      eventually cause the writing processes to block unless we set
      O_NONBLOCK on the write descriptor */
   fcntl(1, F_SETFL, O_NDELAY);
   fcntl(2, F_SETFL, O_NDELAY);
#else
   freopen(LOGFILE, "a", stdout);
   freopen(ERRFILE, "a", stderr);
#endif

   printf("CALVIN %s, patchlevel %d\n", version, PATCHLEVEL);
   fflush(stdout);

#ifdef SYSLOG
   syslog(LOG_INFO,"listener starting up");
#endif

   detach();

   for(i=0,pr=prog; i< num_progs; i++,pr++){
      /* create one socket per port */
      if ((pr->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	 fprintf(stderr, "Socket create failed on socket %d\n", i);
	 perror("ntstart'socket");
	 pr->sock = -1;
#ifdef SYSLOG
         syslog(LOG_ERR,"cannot open socket");
#endif
	 continue;
      }
      /* allow local address resuse */
      optval = 1;
      if (setsockopt(pr->sock, SOL_SOCKET, SO_REUSEADDR, (char *)&optval, 
	 sizeof(int)) < 0) {
#ifdef SYSLOG
         syslog(LOG_ERR,"Error in startup:setsockopt(reuseaddr)");
#endif
	 perror("ntstart'setsockopt");
      }
      /* set linger on close */
      if (setsockopt(pr->sock, SOL_SOCKET, SO_LINGER, (char *)&ling, 
	 sizeof(ling)) < 0) {
#ifdef SYSLOG
         syslog(LOG_ERR,"Error in startup:setsockopt(linger)");
#endif
	 perror("ntstart'startup:setsockopt(linger)");
      }

      /* create the address that we will bind to */
      addr.sin_family = AF_INET;
      addr.sin_addr.s_addr = INADDR_ANY;	/* wild card */
      addr.sin_port = htons(pr->port);

      j=0;
      while(1){
	 /* try to bind to the address */
	 if (bind(pr->sock, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
	    perror("bind");
	    if(j++ == 10){
	       /*
	       if (shutdown(pr->sock, 2) < 0)
		  perror("ntstart'shutdown");
	       */
	       fprintf(stderr,
		  "Giving up: socket bind failed on port %d (%s)\n", 
		  pr->port, pr->prog);
	       fflush(stderr);
	       close(pr->sock);
	       pr->sock = -1;
#ifdef SYSLOG
	       syslog(LOG_ERR,"Error in startup:bind");
#endif
	       shutdownAll();	/* exit */
	    }
	    sleep(2);
	 }
	 else 
	    break;
      }
      printf("listening on %d, connection will start %s \"%s\" %s %s %s %s\n", 
	       prog[i].port, prog[i].prog, prog[i].progname,
	       prog[i].arg[0], prog[i].arg[1], prog[i].arg[2], prog[i].arg[3]);
      fflush(stdout);
   }

   for (;;) {
      /* listen for a connection on all ports */
      for(i=0,pr=prog; i< num_progs; i++,pr++){
	 if(pr->sock < 0) continue;

	 if (listen(pr->sock, 1) < 0) {
	    perror("ntstart'startup: listen");
	    if (shutdown(pr->sock, 2) < 0)
	       perror("ntstart'shutdown");
	    close(pr->sock);
#ifdef SYSLOG
            syslog(LOG_ERR,"Error in startup:listen");
#endif
	    pr->sock = -1;
	 }
      }
again:;
      FD_ZERO(&accept_fds);
      for(i=0,pr=prog; i< num_progs; i++, pr++){
	 if(pr->sock < 0) continue;
	 FD_SET(pr->sock, &accept_fds);
      }
#ifdef TIMESTAMP
      /* set our pipes also */
      FD_SET(proc_stdout, &accept_fds);
      FD_SET(proc_stderr, &accept_fds);
#endif

      /* block until select indicates accept will complete */
      if((st=select(32, &accept_fds, 0, 0, 0))){

	 if (st <= 0 && errno == EINTR)
	    goto again;

#ifdef TIMESTAMP
	 if(FD_ISSET(proc_stdout, &accept_fds))
	    output_timestamp_diag(lfo, proc_stdout);
	 if(FD_ISSET(proc_stderr, &accept_fds))
	    output_timestamp_diag(efo, proc_stderr);
#endif

	 for(i=0,pr=prog; i< num_progs; i++,pr++){
	    if(pr->sock < 0) continue;
	    if(FD_ISSET(pr->sock, &accept_fds)){

	       /* found a connection, procede */
	       len = sizeof(naddr);

	       if ((sock = accept(pr->sock, (struct sockaddr *)&naddr, 
		  &len)) < 0) {
		  perror("ntstart'startup: accept");
		  if (shutdown(pr->sock, 2) < 0)
		     perror("ntstart'shutdown");
		  close(pr->sock);
		  pr->sock = -1;
#ifdef SYSLOG
                  syslog(LOG_ERR,"Error in startup:accept");
#endif
		  continue;
	       }
	       hp = gethostbyaddr((char *) &naddr.sin_addr.s_addr,
				  sizeof(long), AF_INET);
	       if (hp != NULL)
		  (void) strcpy(host_name, hp->h_name);
	       else
		  (void) strcpy(host_name, inet_ntoa(naddr.sin_addr));

#ifdef TIMESTAMP
	       printf("ntstart: connection from %s to %d (%s)\n", 
			host_name, pr->port, pr->prog);
#else
	       time(&curtime);
	       printf("ntstart: connection from %s to %d (%s) at %s", 
			host_name, pr->port, pr->prog,
			ctime(&curtime));
#endif

#ifdef SYSLOG
               syslog(LOG_NOTICE,"player connection from %s",host_name);
#endif
	       fflush(stdout);

	       pid = fork();
	       if (pid < 0) {
		  perror("ntstart'fork");
		  fflush(stderr);
#ifdef SYSLOG
                   syslog(LOG_ERR,"Error in ntstart:fork");
#endif
		  /* If this fails, this connection is out of luck,
                     but the next time we might have enough processes */
		  /* exit(1) */
	       }
	       if (pid == 0) {
		  register struct progrecord	*pr2;
		  /* shutdown all open sockets */
		  for(j=0,pr2=prog; j< num_progs; j++,pr2++){
		     if(pr2->sock < 0) continue;
		     close(pr2->sock);
		  }
		  dup2(sock, 0);
		  /* programs started here must use read()/write() on 
		     descriptor 0 */

		  /* yech */
		  if(*pr->arg[0] && *pr->arg[1] && *pr->arg[2] && *pr->arg[3])
		     execl(pr->prog, pr->progname, pr->arg[0], pr->arg[1],
						   pr->arg[2], pr->arg[3], 
		     				   host_name, 0);
		  else if(*pr->arg[0] && *pr->arg[1] && *pr->arg[2])
		     execl(pr->prog, pr->progname, pr->arg[0], pr->arg[1],
						   pr->arg[2], 
		     				   host_name, 0);
		  else if(*pr->arg[0] && *pr->arg[1])
		     execl(pr->prog, pr->progname, pr->arg[0], pr->arg[1],
		     				   host_name, 0);
		  else if(*pr->arg[0])
		     execl(pr->prog, pr->progname, pr->arg[0],
		     				   host_name, 0);
		  else
		     execl(pr->prog, pr->progname, host_name, 0);

		  _exit(0);
	       }
	       close(sock);
	    }
	 }
      }
   }
}

/*
 * This routine was taken from the daemonII source code in the ipcnetrekII
 * game.  Is it needed? 
 */
void
detach()
{
   int             fd;

   fd = open("/dev/tty", O_RDWR, 0);
   if (fd < 0)
      return;
#ifndef SYSV
   (void) ioctl(fd, TIOCNOTTY, (char *) NULL);
#endif
   (void) close(fd);
}

/*
 * signal handler for SIGCHLD to clean up ntservs
 */
void
handler()
{
   /* since we have wait3 under SunOS, this is recommended */
   while (wait3((union wait *) 0, WNOHANG, (struct rusage *) 0) > 0);
}

void 
shutdownAll()
{
   register struct progrecord	*pr;
   register			i;

   fprintf(stderr, "ntstart: forced exit.\n");
   for(i=0,pr=prog; i< num_progs; i++,pr++){
      if(pr->sock < 0) continue;

      (void) shutdown(pr->sock, 2);
      (void) close(pr->sock);
   }
   exit(0);
}

void
read_portfile()

{
   FILE	*fi = fopen(PORTFILE, "r");
   char	buf[BUFSIZ];
   int	i=0, p;
   struct stat     sbuf;

   if(!fi){
      perror(PORTFILE);
      exit(1);
   }

   while(fgets(buf, BUFSIZ-1, fi)){
      if(sscanf(buf, "%d %s \"%[^\"]\" %s %s %s %s", &p,
				     prog[i].prog,
				     prog[i].progname,
				     prog[i].arg[0],
				     prog[i].arg[1],
				     prog[i].arg[2],
				     prog[i].arg[3]) >= 3){
	 prog[i].port = (unsigned short)p;
	 prog[i].sock = -1;
	 if (stat(prog[i].prog, &sbuf) < 0 || !(sbuf.st_mode & S_IRWXU)) {
	    fprintf(stderr, "\"%s\" not accessible or not executable.\n",
	       prog[i].prog);
	    fflush(stderr);
	 }
	 else{
	    i++;
	 }
      }
   }
   fclose(fi);

   num_progs = i;
}

#ifdef TIMESTAMP
void
output_timestamp_diag(outfile, in)
   
   FILE		*outfile;
   int		in;
{
   struct timeval	tv;
   struct tm		*tp;
   char			buf[BUFSIZ];
   register
   char			*ap, *s;
   int			i;

   /* unbuffered (trouble if we use fdopen) */
   i = read(in, buf, BUFSIZ);
   if(i < 0)
      perror("read");
   if(!i) return;

   gettimeofday(&tv, NULL);
   tp = localtime(&tv.tv_sec);
   ap = asctime(tp);
   if((s=strrchr(ap, '\n')))
      *s = '\0';	/* this is ok since the string is overwritten by
			   asctime each time */

   s = buf; s[i] = 0;
   do {
      fprintf(outfile, "%s: ", ap);
      while(*s){
	 fputc(*s, outfile);
	 if(*s == '\n')
	    break;
	 s++;
      }
      if(*s)
	 s++;
   } while (*s);

   fflush(outfile);
}
#endif

#ifdef SERVICES
void
start_services()
{
   FILE           *fi = fopen(SERVICEFILE, "r");
   char            buf[256];
   int             pid;

   if (!fi)
      return;

   while (fgets(buf, sizeof(buf) - 2, fi)) {
      buf[strlen(buf) - 1] = '\0';
      if ((pid = fork()) == 0) {
	 execl("/bin/sh", "sh", "-c", buf, NULL);
	 perror(buf);
	 _exit(1);
      } else if (pid == -1) {
	 perror("start_services'fork");
      } else
	 wait(NULL);		/* .services must be started with & */
   }
}
#endif
