#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <errno.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include "defs.h"
#include "struct.h"

#if defined(__STDC__) || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* watchmes.c */
void openmem P_((void));
void log P_((struct message *m));
int instr P_((char *string1, char *string2));

#ifdef USE_SYS_PROTOS
extern int                    shmget P_((key_t, int, int));
extern void                   perror P_((const char *));
extern int                    fprintf P_((FILE *, const char *, ...));
extern void *                 shmat P_((int, void *, int));
extern int                    printf P_((const char *, ...));
extern int                    fflush P_((FILE *));
extern unsigned int           sleep P_((unsigned int));
extern time_t                 time P_((time_t *));
#endif

#undef P_

struct player *players;
struct player *me;
struct torp *torps;
struct ship *myship;
struct stats *mystats;
struct planet *planets;
struct phaser *phasers;
struct message *messages;
struct mctl *mctl;
struct team *teams;

void
openmem()
{
    extern int errno;
    int	shmemKey = PKEY;
    int	shmid;
    struct memory	*sharedMemory;

    errno = 0;
    shmid = shmget(shmemKey, 0, 0);
    if (shmid < 0) {
	if (errno != ENOENT) {
	    fprintf(stderr, "shmget\n");
	    exit(1);
	}
	shmid = shmget(shmemKey, 0, 0);
	if (shmid < 0) {
	    fprintf(stderr, "watchmes: Daemon not running\n");
	    exit (1);
	}
    }
    sharedMemory = (struct memory *) shmat(shmid, 0, 0);
    if (sharedMemory == (struct memory *) -1) {
	fprintf(stderr, "shared memory\n");
	exit (1);
    }
    players = sharedMemory->players;
    torps = sharedMemory->torps;
    planets = sharedMemory->planets;
    phasers = sharedMemory->phasers;
    mctl = sharedMemory->mctl;
    messages = sharedMemory->messages;
    teams = sharedMemory->teams;
}

void
main(argc, argv)
int argc;
char *argv[];
{
    int i; 
    int oldmctl;

    int filter = 0;		/* 7/29/91 TC */

    openmem();

    filter = argc > 1;		/* special logging */

    oldmctl=mctl->mc_current;
    if (!filter)
	for (i=0; i<=oldmctl; i++) {
	    printf("%s\n", messages[i].m_data);
	}
    for (;;) {
	sleep(1);
	while (oldmctl!=mctl->mc_current) {
	    oldmctl++;
	    if (oldmctl==50) oldmctl=0;
	    if (filter)
		log(&(messages[oldmctl]));
	    else {
		printf("%s\n", messages[oldmctl].m_data);
		fflush(stdout);
	    }
	}
    }
}

void
log(m)
struct message *m;
{
    long curtime;
    struct tm *tmstruct;
    int hour;

    /* decide whether or not to log this message */

    /* kludge */
   if ((strncmp(m->m_data, "GOD->ALL", 8)==0 && 
          (instr(m->m_data, "was kill") ||
          instr(m->m_data, "killed by"))) || 
          (*m->m_data!=' ' && instr(m->m_data, "We are being attacked")))
	return;

    time(&curtime);
    tmstruct = localtime(&curtime);
    if (!(hour = tmstruct->tm_hour%12)) hour = 12;
    printf("%02d:%02d %-73.73s\n", hour, tmstruct->tm_min,
	   m->m_data);
    fflush(stdout);
}

int
instr(string1, string2)
char *string1, *string2;
{
    char *s;
    int length;

    length=strlen(string2);
    for (s=string1; *s!=0; s++) {
        if (*s == *string2 && strncmp(s, string2, length)==0) return(1);
    }
    return(0);
}
